# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from imio.pm.wsclient import PMMessageFactory as _PM
from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import ALREADY_SENT_TO_PM_ERROR
from imio.pm.wsclient.config import CORRECTLY_SENT_TO_PM_INFO
from imio.pm.wsclient.config import NO_PROPOSING_GROUP_ERROR
from imio.pm.wsclient.config import NO_USER_INFOS_ERROR
from imio.pm.wsclient.config import SEND_WITHOUT_SUFFICIENT_FIELD_MAPPINGS_DEFINED_WARNING
from imio.pm.wsclient.config import TAL_EVAL_FIELD_ERROR
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from imio.pm.wsclient.events import SentToPMEvent
from imio.pm.wsclient.events import WillbeSendToPMEvent
from imio.pm.wsclient.interfaces import IRedirect
from plone import api
from plone.z3cform.layout import wrap_form
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage
from unidecode import unidecode
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.contentprovider import ContentProviders
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IFieldsAndContentProvidersForm
from zope import interface
from zope import schema
from zope.annotation import IAnnotations
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import getAdapter
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.component.hooks import getSite
from zope.contentprovider.provider import ContentProviderBase
from zope.event import notify
from zope.filerepresentation.interfaces import IRawReadFile
from zope.i18n import translate
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory

import base64
import logging


try:
    from collections import OrderedDict
except ImportError:
    # Python < 2.7 compatibility
    from ordereddict import OrderedDict


logger = logging.getLogger('imio.pm.wsclient')


class ISendToPloneMeeting(interface.Interface):
    meetingConfigId = schema.TextLine(title=_(u"Meeting config id"))
    proposingGroup = schema.Choice(title=_PM(u"PloneMeeting_label_proposingGroup"),
                                   description=_(u"Select the proposing group to use "
                                                 "for the created item in PloneMeeting"),
                                   required=True,
                                   vocabulary=u'imio.pm.wsclient.proposing_groups_for_user_vocabulary')
    category = schema.Choice(title=_PM(u"PloneMeeting_label_category"),
                             description=_(u"Select the category to use for the created item item in PloneMeeting"),
                             required=True,
                             vocabulary=u'imio.pm.wsclient.categories_for_user_vocabulary')
    preferredMeeting = schema.Choice(title=_PM(u"PloneMeeting_label_preferredMeeting"),
                                     description=_(u"Select the desired meeting date for the created "
                                                   u"item in PloneMeeting"),
                                     required=False,
                                     vocabulary=u'imio.pm.wsclient.possible_meetingdates_vocabulary')
    annexes = schema.List(title=_PM(u"annexes"),
                          description=_(u"Select the annexes to send"),
                          required=False,
                          value_type=schema.Choice(vocabulary=u'imio.pm.wsclient.annexes_for_user_vocabulary'))


class DisplayDataToSendProvider(ContentProviderBase):
    """
    """
    template = ViewPageTemplateFile('templates/display_data_to_send.pt')

    def __init__(self, context, request, view):
        super(DisplayDataToSendProvider, self).__init__(context, request, view)
        self.__parent__ = view

    def getDisplayableData(self):
        """
          Returns data to be displayed in the resume form
          Do not display :
          - externalIdentifier;
          - empty values.
          Prepare extraAttrs and annexes to be displayed correctly.
        """
        data = self.__parent__.form._buildDataDict()
        if 'externalIdentifier' in data:
            data.pop('externalIdentifier')
        if 'annexes' in data:
            data.pop('annexes')
        for elt in data:
            # remove empty data but keep category and proposingGroup even if empty
            value = isinstance(data[elt], str) and data[elt].strip() or data[elt]
            if not value and elt not in ['category', 'proposingGroup', ]:
                data.pop(elt)
                continue

            if elt == 'extraAttrs':
                res = ['<fieldset><legend>{0}</legend>{1}</fieldset>'.format(
                    translate('PloneMeeting_label_' + extraAttr['key'],
                              domain="PloneMeeting",
                              context=self.request).encode('utf-8'),
                    extraAttr['value'].encode('utf-8')) for extraAttr in data[elt]]
                data[elt] = '<br />'.join(res)

        if 'title' not in data:
            IStatusMessage(self.request).addStatusMessage(_(SEND_WITHOUT_SUFFICIENT_FIELD_MAPPINGS_DEFINED_WARNING),
                                                          'warning')
        return data

    def render(self):
        return self.template()


class SendToPloneMeetingForm(form.Form):
    implements(IFieldsAndContentProvidersForm)

    fields = field.Fields(ISendToPloneMeeting)
    fields['annexes'].widgetFactory = CheckBoxFieldWidget

    ignoreContext = True  # don't use context to get widget data

    contentProviders = ContentProviders()
    contentProviders['dataToSend'] = DisplayDataToSendProvider
    # put the 'dataToSend' in last position
    contentProviders['dataToSend'].position = 3
    label = _(u"Send to PloneMeeting")
    description = u''
    _finishedSent = False
    _displayErrorsInOverlay = False

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.meetingConfigId = self._findMeetingConfigId()
        self.proposingGroupId = self.request.form.get('form.widgets.proposingGroupId', [''])[0]
        self.portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        self.portal = self.portal_state.portal()
        self.ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        # manage the label to display to wich meetingConfig we are sending the element...
        self.label = translate('Send to',
                               domain='imio.pm.wsclient',
                               mapping={'meetingConfigTitle':
                                        self.ws4pmSettings.getMeetingConfigTitle(self.meetingConfigId),
                                        },
                               context=self.request)

    @button.buttonAndHandler(_('Send'), name='send_to_plonemeeting')
    def handleSendToPloneMeeting(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.proposingGroupId = self.request.form.get('form.widgets.proposingGroup')[0]
        # do send to PloneMeeting
        self._doSendToPloneMeeting()

    @button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self._finishedSent = True

    def update(self):
        """ """
        # use checkAlreadySentToPloneMeeting that will return :
        # None if could not connect
        # True if already sent
        # False if not already sent, in this case we can proceed...
        alreadySent = self.ws4pmSettings.checkAlreadySentToPloneMeeting(self.context, (self.meetingConfigId,))
        settings = self.ws4pmSettings.settings()
        if alreadySent and settings.only_one_sending:
            IStatusMessage(self.request).addStatusMessage(_(ALREADY_SENT_TO_PM_ERROR), "error")
            self._changeFormForErrors()
            return
        else:
            # None means that it was already sent but that it could not connect to PloneMeeting
            # False means that is was not sent, so no connection test is made to PloneMeeting for performance reason
            if alreadySent is not None:
                # now connect to PloneMeeting
                client = self.ws4pmSettings._soap_connectToPloneMeeting()
            if alreadySent is None or not client:
                IStatusMessage(self.request).addStatusMessage(_(UNABLE_TO_CONNECT_ERROR), "error")
                self._changeFormForErrors()
                return

        # do not go further if current user can not create an item in
        # PloneMeeting with any proposingGroup
        userInfos = self.ws4pmSettings._soap_getUserInfos(showGroups=True, suffix='creators')
        if not userInfos or 'groups' not in userInfos:
            userThatWillCreate = self.ws4pmSettings._getUserIdToUseInTheNameOfWith()
            if not userInfos:
                IStatusMessage(self.request).addStatusMessage(
                    _(NO_USER_INFOS_ERROR, mapping={'userId': userThatWillCreate}), "error")
            else:
                IStatusMessage(self.request).addStatusMessage(
                    _(NO_PROPOSING_GROUP_ERROR, mapping={'userId': userThatWillCreate}), "error")
            self._changeFormForErrors()
            return

        # now that we can connect to the webservice, check that the user can actually trigger that action
        # indeed parameters are sent thru the request, and so someone could do nasty things...
        # check that the real currentUrl is on available in object_buttons actions for the user
        availableActions = self.portal.portal_actions.listFilteredActionsFor(self.context).get('object_buttons', [])
        # rebuild real url called by the action
        currentUrl = unicode(self.request['ACTUAL_URL'] + '?meetingConfigId=' + self.meetingConfigId, 'utf-8')
        # now check if this url is available in the actions for the user
        mayDoAction = False
        for action in availableActions:
            if action['url'] == currentUrl:
                mayDoAction = True
                break
        if not mayDoAction:
            raise Unauthorized

        super(SendToPloneMeetingForm, self).update()
        # while calling parent's update, vocabularies are initialized
        # if there where errors in the vocabularies, hide the form
        if self.request.get('error_in_vocabularies', False):
            self._changeFormForErrors()
        # after calling parent's update, self.actions are available
        self.actions.get('cancel').addClass('standalone')

    def updateWidgets(self):
        portal = getSite()
        # this is also called by the kss inline_validation, avoid too much work...
        if not portal.__module__ == 'Products.CMFPlone.Portal':
            return
        # hide the meetingConfigId field
        self.fields.get('meetingConfigId').mode = HIDDEN_MODE
        # hide the widget if the linked vocabulary is empty, it means that
        # the linked meetingConfig does not use categories...
        # hide it and set it to required=False
        category_field = self.fields.get('category')
        category_field.mode = not bool(self._getCategoriesVocab()) and HIDDEN_MODE or None
        # as category is a required field, hidding it is not enough...
        category_field.field.required = bool(self._getCategoriesVocab()) and True or False
        # XXX manipulate self.fields BEFORE doing form.Form.updateWidgets
        form.Form.updateWidgets(self)
        # hide annexes if nothing in vocabulary
        annexes_wdg = self.widgets.get('annexes')
        if len(annexes_wdg.terms) == 0:
            annexes_wdg.mode = HIDDEN_MODE
        # add a 'Choose a value...'
        self.widgets.get('proposingGroup').prompt = True
        self.widgets.get('category').prompt = True
        # initialize the value of the meetingConfigId field to what is found in the request
        self.widgets.get('meetingConfigId').value = self._findMeetingConfigId()
        # add the class 'standalone' on the 'Cancel' button

    def render(self):
        if self._finishedSent:
            IRedirect(self.request).redirect(self.context.absolute_url())
            return ""
        return super(SendToPloneMeetingForm, self).render()

    def _findMeetingConfigId(self):
        """
          Find the meetingConfigId wherever it is...
        """
        return self.request.get('meetingConfigId', '') or \
            self.request.form.get('form.widgets.meetingConfigId')

    def _doSendToPloneMeeting(self):
        """
          The method actually called while sending to PloneMeeting
        """
        # check again if already sent before sending
        # this avoid double sent from 2 opened form to send
        settings = self.ws4pmSettings.settings()
        if self.ws4pmSettings.checkAlreadySentToPloneMeeting(self.context, (self.meetingConfigId,)) and \
                settings.only_one_sending:
            return False
        # build the creationData
        client = self.ws4pmSettings._soap_connectToPloneMeeting()
        creation_data = self._getCreationData(client)

        notify(WillbeSendToPMEvent(self.context))

        # call the SOAP method actually creating the item
        res = self.ws4pmSettings._soap_createItem(self.meetingConfigId,
                                                  self.proposingGroupId,
                                                  creation_data)
        if res:
            uid, warnings = res
            self.request.set('show_send_to_pm_form', False)
            self.portal.plone_utils.addPortalMessage(_(CORRECTLY_SENT_TO_PM_INFO), 'info')
            if warnings:
                for warning in warnings:
                    # show warnings in the web interface and add it to the Zope log
                    logger.warning(warning)
                    IStatusMessage(self.request).addStatusMessage(_(warning), 'warning')
            # finally save in the self.context annotation that the item has been sent
            annotations = IAnnotations(self.context)
            if WS4PMCLIENT_ANNOTATION_KEY not in annotations:
                annotations[WS4PMCLIENT_ANNOTATION_KEY] = [self.meetingConfigId, ]
            else:
                # do not use .append directly on the annotations or it does not save
                # correctly and when Zope restarts, the added annotation is lost???
                existingAnnotations = list(annotations[WS4PMCLIENT_ANNOTATION_KEY])
                existingAnnotations.append(self.meetingConfigId)
                annotations[WS4PMCLIENT_ANNOTATION_KEY] = existingAnnotations
            self._finishedSent = True

            notify(SentToPMEvent(self.context))

            return True
        return False

    def _getCreationData(self, client):
        """
          Build creationData dict that will be used to actually create
          the item in PloneMeeting thru SOAP createItem call
        """
        data = self._buildDataDict()
        # now that every values are evaluated, build the CreationData
        creation_data = client.factory.create('CreationData')
        for elt in data:
            # proposingGroup is managed apart
            if elt == u'proposingGroup':
                continue
            if isinstance(data[elt], str):
                data[elt] = unicode(data[elt], 'utf-8')
            if elt == 'extraAttrs':
                for attr in data[elt]:
                    attr['value'] = safe_unicode(attr['value'])
            creation_data[elt] = data[elt]
        # initialize the externalIdentifier to the context UID
        creation_data['externalIdentifier'] = self.context.UID()
        return creation_data

    def _buildDataDict(self):
        """
        """
        data = OrderedDict()
        settings = self.ws4pmSettings.settings()
        # initialize annexes field from the form, not the field_mappings
        data['annexes'] = self._buildAnnexesData()
        data['preferredMeeting'] = self.request.form.get('form.widgets.preferredMeeting', [u'', ])[0]
        # if preferredMeeting is '--NOVALUE--',  remove it from data to send
        if data['preferredMeeting'] == '--NOVALUE--':
            data.pop('preferredMeeting')
        # initialize category field in case it is not defined in field_mappings
        data['category'] = self.request.form.get('form.widgets.category', [u'', ])[0]
        # if category is '--NOVALUE--', consider it empty
        if data['category'] == '--NOVALUE--':
            data['category'] = ''
        for availableData in settings.field_mappings:
            field_name = availableData['field_name']
            if field_name == 'category':
                # check that the meetingConfig we want to send the item to
                # actually use categories
                if not self._getCategoriesVocab():
                    data[field_name] = u''
                else:
                    continue
            expr = availableData['expression']
            # make the meetingConfigId available in the expression
            vars = {}
            vars['meetingConfigId'] = self.meetingConfigId
            vars['proposingGroupId'] = self.proposingGroupId
            # evaluate the expression
            try:
                data[field_name] = self.ws4pmSettings.renderTALExpression(self.context,
                                                                          self.portal,
                                                                          expr,
                                                                          vars)
            except Exception, e:
                IStatusMessage(self.request).addStatusMessage(
                    _(TAL_EVAL_FIELD_ERROR, mapping={'expr': expr,
                                                     'field_name': field_name,
                                                     'error': e}),
                    "error")
                return self.request.RESPONSE.redirect(self.context.absolute_url())
        return data

    def _buildAnnexesData(self):
        catalog = api.portal.get_tool('portal_catalog')
        annexes_data = []
        selected_annexes = self.request.form.get('form.widgets.annexes', [])
        for selected_annex in selected_annexes:
            annex_brains = catalog.unrestrictedSearchResults(UID=selected_annex)
            if annex_brains:
                annex = annex_brains[0].getObject()
                annex_file = getAdapter(annex, IRawReadFile)
                annex_name = type(annex_file.name) in [unicode] and annex_file.name or annex_file.name.decode('utf-8')
                annexes_data.append(
                    {
                        'title': unidecode(annex.title.decode('utf-8')),
                        'filename': unidecode(annex_name),
                        'file': base64.b64encode(annex_file.read()),
                    }
                )
        return annexes_data

    def _getCategoriesVocab(self):
        """Return the vocabulary used for the categories field"""
        portal = getSite()
        factory = queryUtility(IVocabularyFactory, u'imio.pm.wsclient.categories_for_user_vocabulary')
        return factory(portal)

    def _getProposingGroupsVocab(self):
        """Return the vocabulary used for the proposingGroup field"""
        portal = getSite()
        factory = queryUtility(IVocabularyFactory, u'imio.pm.wsclient.proposing_groups_for_user_vocabulary')
        return factory(portal)

    def _changeFormForErrors(self):
        """
        """
        # if we use an overlay popup do some nasty things...
        if 'ajax_load' in self.request.form:
            self.template = ViewPageTemplateFile('templates/show_errors_in_overlay_form.pt').__get__(self, '')
        else:
            self._finishedSent = True
        super(SendToPloneMeetingForm, self).update()


SendToPloneMeetingWrapper = wrap_form(SendToPloneMeetingForm)



# -*- coding: utf-8 -*-

from zope.filerepresentation.interfaces import IRawReadFile
from zope.interface import implements


class ATRawReadFile(object):
    """
    """
    implements(IRawReadFile)

    encoding = 'utf-8'
    name = None

    def __init__(self, context):
        self.context = context
        self._size = 0

    def size(self):
        stream = self._getStream()
        pos = stream.tell()
        stream.seek(0, 2)
        size = stream.tell()
        stream.seek(pos)
        return size

    def read(self, size=None):
        if size is not None:
            return self._getStream().read(size)
        else:
            return self._getStream().read()

    def readline(self, size=None):
        if size is None:
            return self._getStream().readline()
        else:
            return self._getStream().readline(size)

    def readlines(self, sizehint=None):
        if sizehint is None:
            return self._getStream().readlines()
        else:
            return self._getStream().readlines(sizehint)

    def _getStream(self):
        return self.context.getFile().getIterator()

    def __iter__(self):
        return self

    def next(self):
        return self._getStream().next()

    @property
    def encoding(self):
        return self._getMessage().get_charset() or 'utf-8'

    @property
    def name(self):
        return self.context.getFilename()



# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# Copyright (c) 2013 by Imio
#
# GNU General Public License (GPL)
#

from plone import api

import logging


logger = logging.getLogger('imio.pm.wsclient: setuphandlers')

__author__ = """Gauthier Bastien <gauthier@imio.be>"""
__docformat__ = 'plaintext'


def isNotImioPmWsClientProfile(context):
    return context.readDataFile("imio_pm_wsclient_marker.txt") is None


def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotImioPmWsClientProfile(context):
        return


def reload_js_registry(context):
    setup_tool = api.portal.get_tool('portal_setup')
    setup_tool.runImportStepFromProfile('profile-imio.pm.wsclient:default', 'jsregistry')



# -*- coding: utf-8 -*-
#
# File: testViewlets.py
#
# GNU General Public License (GPL)
#

from datetime import datetime
from dateutil import tz
from imio.pm.wsclient.browser.viewlets import PloneMeetingInfosViewlet
from imio.pm.wsclient.config import CAN_NOT_SEE_LINKED_ITEMS_INFO
from imio.pm.wsclient.config import CORRECTLY_SENT_TO_PM_INFO
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import cleanMemoize
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from Products.statusmessages.interfaces import IStatusMessage
from zope.annotation import IAnnotations

import transaction


class testViewlets(WS4PMCLIENTTestCase):
    """
        Tests the browser.testViewlets methods
    """

    def test_available(self):
        """ """
        self.changeUser('admin')
        document = createDocument(self.portal)
        # by default, no viewlet_display_condition TAL expression
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        settings = viewlet.ws4pmSettings.settings()
        settings.viewlet_display_condition = u''
        # no items created/linked, so the viewlet is not displayed
        self.assertFalse(viewlet.available())
        # viewlet is displayed depending on :
        # by default, the fact that it is linked to an item
        # or if a TAL expression is defined in the config and returns True
        # test with a defined TAL expression in the configuration
        settings.viewlet_display_condition = u'python: True'
        # if a TAL expression is defined, it take precedence
        # available is memoized so it is still False...
        self.assertFalse(viewlet.available())
        # remove memoized informations
        cleanMemoize(self.request, viewlet)
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        # now remove the TAL expression and send the object to PloneMeeting
        settings.viewlet_display_condition = u''
        self.assertFalse(viewlet.available())
        cleanMemoize(self.request, viewlet)
        item = self._sendToPloneMeeting(document)
        # now that the element has been sent, the viewlet is available
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        # define a TAL expression taking care of the 'isLinked'
        settings.viewlet_display_condition = u'python: isLinked and object.portal_type == "wrong_value"'
        self.assertFalse(viewlet.available())
        cleanMemoize(self.request, viewlet)
        settings.viewlet_display_condition = u'python: isLinked and object.portal_type == "Document"'
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        # if the TAL expression has errors, available is False and a message is displayed
        messages = IStatusMessage(self.request)
        # by default, 2 messages already exist, these are item creation related messages
        shownMessages = messages.show()
        self.assertTrue(len(shownMessages) == 1)
        self.assertTrue(shownMessages[0].message, CORRECTLY_SENT_TO_PM_INFO)
        settings.viewlet_display_condition = u'python: object.getUnexistingAttribute()'
        # in case there is a problem, a message is displayed in a tuple (msg, error_level)
        self.assertTrue(isinstance(viewlet.available(), tuple))
        cleanMemoize(self.request, viewlet)
        # now check when the linked item is removed
        settings.viewlet_display_condition = u''
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        item.aq_inner.aq_parent.manage_delObjects(ids=[item.getId(), ])
        transaction.commit()
        self.assertFalse(viewlet.available())

    def test_getPloneMeetingLinkedInfos(self):
        """
          Test the getPloneMeetingLinkedInfos method :
          - if nothing found, an empty {} is returned
          - if elements are found, relevant data are returned
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available() is False)
        # now send an element to PloneMeeting and check again
        cleanMemoize(self.request, viewlet)
        item = self._sendToPloneMeeting(document)
        # we received informations about the created item
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())

    def test_canNotSeeLinkedInfos(self):
        """
          If the element has been sent to PloneMeeting but current user can not see these
          items in PloneMeeting, a message is displayed
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        item = self._sendToPloneMeeting(document)
        # we received informations about the created item
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())
        # admin will create the element inTheNameOf 'pmCreator1'
        self.changeUser('pmCreator1')
        cleanMemoize(self.request, viewlet)
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())
        # 'pmCreator2' will not see the infos about items, just a message
        self.changeUser('pmCreator2')
        cleanMemoize(self.request, viewlet)
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos() == (CAN_NOT_SEE_LINKED_ITEMS_INFO, 'info'))

    def test_canNotConnectTemporarily(self):
        """
          Test a connection problem in PloneMeeting after an item has already been sent
          - connect to PM successfully
          - send the item successfully
          - break settings
          - check what is going on while not being able to show PM related informations
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        # send an element to PloneMeeting
        item = self._sendToPloneMeeting(document)
        # correctly sent
        self.assertTrue(viewlet.available() is True)
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())
        setCorrectSettingsConfig(self.portal, **{'pm_url': u'http://wrong/url'})
        cleanMemoize(self.request, viewlet)
        # no available
        # a message is returned in the viewlet by the viewlet.available method
        self.assertTrue(viewlet.available() == (UNABLE_TO_CONNECT_ERROR, 'error'))
        # the annotations on the document are still correct
        self.assertTrue(IAnnotations(document)[WS4PMCLIENT_ANNOTATION_KEY] == ['plonemeeting-assembly'])

    def test_displayMeetingDate(self):
        """
          Test the displayMeetingDate method :
          - not date, a '-' is returned
          - a date, a formatted version is returned with hours if hours != '00:00'
        """
        viewlet = PloneMeetingInfosViewlet(self.portal, self.request, None, None)
        viewlet.update()
        # The no meeting date correspond to a date in 1950
        self.assertTrue(viewlet.displayMeetingDate(datetime(1950, 1, 1)) == '-')
        # If there is a date, it is corretly displayed
        # if no hours (hours = 00:00), a short format is used, without displaying hours
        # the sent date is UTC but displayMeetingDate will return it using local timezone
        # so build a date that will return a date without hours...
        utcMeetingDate = datetime(2013, 6, 10)
        utcMeetingDate = utcMeetingDate.replace(tzinfo=tz.tzlocal())
        delta = utcMeetingDate.utcoffset()
        utcMeetingDate = utcMeetingDate - delta
        # set utcMeetingDate as being UTC
        utcMeetingDate = utcMeetingDate.replace(tzinfo=tz.tzutc())
        self.assertTrue(viewlet.displayMeetingDate(datetime(2013, 6, 10)) == u'Jun 10, 2013')
        # If hours, then a long format is used
        self.assertTrue(viewlet.displayMeetingDate(datetime(2013, 6, 10, 15, 30)) == u'Jun 10, 2013 03:30 PM')

    def test_displayedDateRespectTimeZone(self):
        """
          This confirm a bug in suds 0.4 corrected in suds-jurko 0.4.1.jurko.4
          where returned date does not work when not using current time zone.
          For example, a GMT+1 date is wrongly displayed when we are in GMT+2, it miss 1 hour,
          so 2013/03/03 00:00 becomes 2013/03/02 23:00...
          Returned dates should always be UTC.
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        item = self._sendToPloneMeeting(document)
        # create a meeting and present the item
        self.changeUser('pmManager')
        # use a date in GMT+1
        meeting = self.create('Meeting', date=datetime(2013, 3, 3))
        self.presentItem(item)
        transaction.commit()
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        meeting_date = viewlet.getPloneMeetingLinkedInfos()[0]['meeting_date']
        # date TZ naive, uses UTC
        self.assertEqual(meeting_date.tzname(), 'UTC')
        self.assertEquals((meeting_date.year,
                           meeting_date.month,
                           meeting_date.day,
                           meeting_date.hour,
                           meeting_date.minute),
                          (2013, 3, 2, 23, 0))
        # change meeting date, use a date in GMT+2
        meeting.date = datetime(2013, 8, 8)
        transaction.commit()
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        meeting_date = viewlet.getPloneMeetingLinkedInfos()[0]['meeting_date']
        self.assertEqual(meeting_date.tzname(), 'UTC')
        self.assertEquals((meeting_date.year,
                           meeting_date.month,
                           meeting_date.day,
                           meeting_date.hour,
                           meeting_date.minute),
                          (2013, 8, 7, 22, 0))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testViewlets, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-

from datetime import datetime
from imio.pm.ws.config import POD_TEMPLATE_ID_PATTERN
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import cleanMemoize
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from plone import api
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter


import transaction


class testSOAPMethods(WS4PMCLIENTTestCase):
    """
        Tests the browser.settings SOAP client methods
    """

    def test_soap_connectToPloneMeeting(self):
        """Check that we can actually connect to PloneMeeting with given parameters."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        settings = ws4pmSettings.settings()
        setCorrectSettingsConfig(self.portal, minimal=True)
        # with valid informations, we can connect to PloneMeeting SOAP webservices
        self.failUnless(ws4pmSettings._soap_connectToPloneMeeting())
        # if either url or username/password is not valid, we can not connect...
        valid_url = settings.pm_url
        settings.pm_url = settings.pm_url + 'invalidEndOfURL'
        cleanMemoize(self.request)
        # with invalid url, it fails...
        self.failIf(ws4pmSettings._soap_connectToPloneMeeting())
        settings.pm_url = valid_url
        # with valid url but wrong password, it fails...
        settings.pm_password = u'wrongPassword'
        cleanMemoize(self.request)
        self.failIf(ws4pmSettings._soap_connectToPloneMeeting())

    def test_soap_getConfigInfos(self):
        """Check that we receive valid infos about the PloneMeeting's configuration."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        configInfos = ws4pmSettings._soap_getConfigInfos()
        # check thatt we received elements like MeetingConfig and MeetingGroups
        self.assertTrue(configInfos.configInfo)
        self.assertTrue(configInfos.groupInfo)
        # by default, no categories
        self.assertFalse(hasattr(configInfos.configInfo[0], 'categories'))
        # we can ask categories by passing a showCategories=True to _soap_getConfigInfos
        configInfos = ws4pmSettings._soap_getConfigInfos(showCategories=True)
        self.assertTrue(hasattr(configInfos.configInfo[1], 'categories'))

    def test_soap_getItemCreationAvailableData(self):
        """Check that we receive the list of available data for creating an item."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        availableData = ws4pmSettings._soap_getItemCreationAvailableData()
        availableData.sort()
        self.assertEqual(availableData, ['annexes',
                                          'associatedGroups',
                                          'category',
                                          'decision',
                                          'description',
                                          'detailedDescription',
                                          'externalIdentifier',
                                          'extraAttrs',
                                          'groupsInCharge',
                                          'motivation',
                                          'optionalAdvisers',
                                          'preferredMeeting',
                                          'proposingGroup',
                                          'title',
                                          'toDiscuss'])

    def test_soap_getItemInfos(self):
        """Check the fact of getting informations about an existing item."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        # by default no item exist in the portal...  So create one!
        self.changeUser('pmManager')
        item = self.create('MeetingItem')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        self.assertTrue(len(ws4pmSettings._soap_getItemInfos({'UID': item.UID()})) == 1)
        # getItemInfos is called inTheNameOf the currently connected user
        # if the user (like 'pmCreator1') can see the item, he gets it in the request
        # either (like for 'pmCreator2') the item is not found
        self.changeUser('pmCreator1')
        self.assertTrue(len(ws4pmSettings._soap_getItemInfos({'UID': item.UID()})) == 1)
        self.changeUser('pmCreator2')
        self.assertTrue(len(ws4pmSettings._soap_getItemInfos({'UID': item.UID()})) == 0)

    def test_soap_searchItems(self):
        """Check the fact of searching items informations about existing items."""
        SAME_TITLE = 'sameTitleForBothItems'
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        # Create 2 items, one for 'pmCreator1' and one for 'pmCreator2'
        # items are only viewable by their creator as 'pmCreatorx' not in the same proposingGroup
        self.changeUser('pmCreator1')
        item1 = self.create('MeetingItem')
        item1.setTitle(SAME_TITLE)
        item1.reindexObject(idxs=['Title', ])
        self.changeUser('pmCreator2')
        item2 = self.create('MeetingItem')
        item2.setTitle(SAME_TITLE)
        item2.reindexObject(idxs=['Title', ])
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        # searchItems will automatically restrict searches to the connected user
        self.changeUser('pmCreator1')
        result = ws4pmSettings._soap_searchItems({'Title': SAME_TITLE})
        self.assertTrue(len(result), 1)
        self.assertTrue(result[0].UID == item1.UID())
        self.changeUser('pmCreator2')
        result = ws4pmSettings._soap_searchItems({'Title': SAME_TITLE})
        self.assertTrue(len(result), 1)
        self.assertTrue(result[0].UID == item2.UID())

    def test_soap_createItem(self):
        """Check item creation.
           Item creation will automatically use currently connected user
           to create the item regarding the _getUserIdToUseInTheNameOfWith."""
        cfg2 = self.meetingConfig2
        cfg2Id = cfg2.getId()
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        self.changeUser('pmManager')
        self.setMeetingConfig(cfg2Id)
        test_meeting = self.create('Meeting')
        self.freezeMeeting(test_meeting)
        self.changeUser('pmCreator1')
        # create the 'pmCreator1' member area to be able to create an item
        pmFolder = self.tool.getPloneMeetingFolder(cfg2Id)
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        # create an item for 'pmCreator1'
        data = {'title': u'My sample item',
                'category': u'deployment',
                'description': u'<p>My description</p>',
                # also use accents, this was failing with suds-jurko 0.5
                'decision': u'<p>My d\xe9cision</p>',
                'preferredMeeting': test_meeting.UID(),
                'externalIdentifier': u'my-external-identifier',
                'extraAttrs': [{'key': 'internalNotes',
                                'value': '<p>Internal notes</p>'}]}
        result = ws4pmSettings._soap_createItem(cfg2Id, 'developers', data)
        # commit again so the item is really created
        transaction.commit()
        # the item is created and his UID is returned
        # check that the item is actually created inTheNameOf 'pmCreator1'
        itemUID = result[0]
        item = self.portal.uid_catalog(UID=itemUID)[0].getObject()
        # created in the 'pmCreator1' member area
        self.assertTrue(item.aq_inner.aq_parent.UID(), pmFolder.UID())
        self.assertTrue(item.owner_info()['id'] == 'pmCreator1')
        self.assertEqual(item.Title(), data['title'])
        self.assertEqual(item.getCategory(), data['category'])
        self.assertEqual(item.Description(), data['description'])
        self.assertEqual(item.getDecision(), data['decision'].encode('utf-8'))
        self.assertEqual(item.getPreferredMeeting(), test_meeting.UID(), data['preferredMeeting'])
        self.assertEqual(item.externalIdentifier, data['externalIdentifier'])
        # extraAttrs
        self.assertEqual(item.getInternalNotes(), data['extraAttrs'][0]['value'])

        # if we try to create with wrong data, the SOAP ws returns a response
        # that is displayed to the user creating the item
        data['category'] = 'unexisting-category-id'
        result = ws4pmSettings._soap_createItem('plonegov-assembly', 'developers', data)
        self.assertIsNone(result)
        messages = IStatusMessage(self.request)
        # a message is displayed
        self.assertEqual(messages.show()[-1].message,
                          u"An error occured during the item creation in PloneMeeting! "
                          "The error message was : Server raised fault: ''unexisting-category-id' "
                          "is not available for the 'developers' group!'")

    def test_soap_getItemTemplate(self):
        """Check while getting rendered template for an item.
           getItemTemplate will automatically use currently connected user
           to render item template regarding the _getUserIdToUseInTheNameOfWith."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        # by default no item exist in the portal...  So create one!
        self.changeUser('pmManager')
        item = self.create('MeetingItem')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        self.assertTrue(ws4pmSettings._soap_getItemTemplate(
            {'itemUID': item.UID(),
             'templateId': POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt')}))
        # getItemTemplate is called inTheNameOf the currently connected user
        # if the user (like 'pmCreator1') can see the item, he gets the rendered template
        # either (like for 'pmCreator2') nothing is returned
        self.changeUser('pmCreator1')
        self.assertTrue(ws4pmSettings._soap_getItemTemplate(
            {'itemUID': item.UID(),
             'templateId': POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt')}))
        self.changeUser('pmCreator2')
        self.assertFalse(ws4pmSettings._soap_getItemTemplate(
            {'itemUID': item.UID(),
             'templateId': POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt')}))

    def test_soap_getMeetingAcceptingItems(self):
        """Check getting accepting items meeting.
           Should only return meetings in the state 'creation' and 'frozen'"""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        cfg = self.meetingConfig
        cfgId = cfg.getId()
        meetings = ws4pmSettings._soap_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        self.assertEqual(meetings, [])
        self.changeUser('pmManager')
        meeting_1 = self.create('Meeting', date=datetime(2013, 3, 3))
        meeting_2 = self.create('Meeting', date=datetime(2013, 3, 3))
        transaction.commit()
        self.changeUser('pmCreator1')
        meetings = ws4pmSettings._soap_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        # so far find the two meetings
        self.assertEqual(len(meetings), 2)

        # freeze meeting_2 => it still should be in the accepting items meetings
        self.changeUser('pmManager')
        api.content.transition(meeting_2, 'freeze')
        transaction.commit()
        self.changeUser('pmCreator1')
        meetings = ws4pmSettings._soap_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        self.assertEqual(len(meetings), 2)

        self.changeUser('pmManager')
        api.content.transition(meeting_2, 'publish')
        api.content.transition(meeting_2, 'decide')
        transaction.commit()
        # after publishing meeting_2, it should not be in the results anymore.
        self.changeUser('pmCreator1')
        meetings = ws4pmSettings._soap_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        self.assertEqual(len(meetings), 1)
        self.assertEqual(meetings[0].UID, meeting_1.UID())
        # if no inTheNameOf param is explicitly passed, _soap_getMeetingsAcceptingItems()
        # should set a default one.
        meetings = ws4pmSettings._soap_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId}
        )
        self.assertEqual(len(meetings), 1)
        self.assertEqual(meetings[0].UID, meeting_1.UID())

        # As pmManager, we should get all the meetings
        meetings = ws4pmSettings._soap_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmManager'}
        )
        self.assertEqual(len(meetings), 2)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPMethods, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-
#
# File: testItemMethods.py
#
# Copyright (c) 2012 by CommunesPlone
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from AccessControl import Unauthorized
from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import ALREADY_SENT_TO_PM_ERROR
from imio.pm.wsclient.config import CORRECTLY_SENT_TO_PM_INFO
from imio.pm.wsclient.config import NO_PROPOSING_GROUP_ERROR
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createAnnex
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import SEND_TO_PM_VIEW_NAME
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from Products.statusmessages.interfaces import IStatusMessage
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.i18n import translate

import transaction


class testForms(WS4PMCLIENTTestCase):
    """
        Tests the browser.settings SOAP client methods
    """

    def test_canNotCallFormIfNotConnected(self):
        """If no valid parameters are defined in the settings, the view is not accessible
           and a relevant message if displayed to the member in portal_messages."""
        # only available to connected users
        self.assertRaises(Unauthorized, self.portal.restrictedTraverse, SEND_TO_PM_VIEW_NAME)
        self.changeUser('pmCreator1')
        self.portal.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance

    def test_canNotConnectToPloneMeeting(self):
        """If no valid parameters are defined in the settings, the view is not accessible
           and a relevant message if displayed to the member in portal_messages."""
        # set base params to avoid extra status messages like 'no field_mappings defined'
        setCorrectSettingsConfig(self.portal, setConnectionParams=False, withValidation=False)
        # only available to connected users
        self.changeUser('pmCreator1')
        self._configureRequestForView(self.portal)
        view = self.portal.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        # when we can not connect, a message is displayed to the user
        messages = IStatusMessage(self.request)
        self.assertTrue(len(messages.show()) == 0)
        # call the view
        view()
        self.assertTrue(len(messages.show()) == 1)
        self.assertEquals(messages.show()[0].message, UNABLE_TO_CONNECT_ERROR)

    def test_canNotExecuteWrongAction(self):
        """While calling the view to execute an action, we check if the user can actually
           execute the action regarding the parameters defined in settings.generated_actions."""
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        # build an url that should not be accessible by the user
        # if the url does not correspond to an available wsclient linked action,
        # an Unauthorized is raised.  This make sure the triggered action is
        # available to the user
        self.request.set('URL', document.absolute_url())
        self.request.set('ACTUAL_URL', document.absolute_url() + '/%s' % SEND_TO_PM_VIEW_NAME)
        self.request.set('meetingConfigId', 'wrong-meeting-config-id')
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        self.assertRaises(Unauthorized, view)

    def test_sendItemToPloneMeeting(self):
        """Test that the item is actually sent to PloneMeeting."""
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        self._configureRequestForView(document)
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        # create an annex to send...
        annex = createAnnex(self.portal.Members.pmCreator1)
        # before sending, no item is linked to the document
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        self.assertTrue(len(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})) == 0)
        # create the 'pmCreator1' member area to be able to create an item
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator1')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created member area...
        transaction.commit()
        # send to PloneMeeting
        # as form.button.Send is not in the form, nothing is done but returning the views's index
        # the form for sending an element is displayed
        form_action = '<form class="rowlike enableUnloadProtection   kssattr-formname-send_to_plonemeeting_form"' \
            ' action="{0}/Members/pmCreator1/document" method="post"' \
            ' id="form" enctype="multipart/form-data">'.format(self.portal.absolute_url())
        self.assertTrue(form_action in view())
        self.assertTrue(len(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})) == 0)
        # now send the element to PM
        view.proposingGroupId = 'developers'
        view.request.form['form.widgets.annexes'] = [annex.UID()]
        # view._doSendToPloneMeeting returns True if the element was actually sent
        self.assertTrue(view._doSendToPloneMeeting())
        # while the element is sent, the view will return nothing...
        self.assertFalse(view())
        # now that the element has been sent, an item is linked to the document
        items = ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})
        self.assertTrue(len(items) == 1)
        # moreover, as defined in the configuration, 1 annex were added to the item
        itemInfos = ws4pmSettings._soap_getItemInfos({'UID': items[0]['UID'], 'showAnnexes': True})[0]
        self.assertTrue(len(itemInfos['annexes']) == 1)

    def test_canNotSendIfInNoPMCreatorGroup(self):
        """
          If the user that wants to send the item in PloneMeeting is not a creator in PM,
          aka is not in a _creators suffixed group, a message is displayed to him.
        """
        # remove pmCreator2 from the vendors_creators group
        # first check that the user is actually in a _creators group
        pmCreator2 = self.portal.portal_membership.getMemberById('pmCreator2')
        self.assertTrue([group for group in self.portal.acl_users.source_groups.getGroupsForPrincipal(pmCreator2)
                         if group.endswith('_creators')])
        self.portal.portal_groups.removePrincipalFromGroup('pmCreator2', self.vendors_creators)
        # pmCreator2 is no more in a _creators group
        self.assertFalse([group for group in self.portal.acl_users.source_groups.getGroupsForPrincipal(pmCreator2)
                         if group.endswith('_creators')])
        # try to send the item
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator2')
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator2')
        transaction.commit()
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator2)
        messages = IStatusMessage(self.request)
        # if no item is created, _sendToPloneMeeting returns None
        self.assertFalse(self._sendToPloneMeeting(document, user='pmCreator2', proposingGroup=self.vendors_uid))
        msg = _(NO_PROPOSING_GROUP_ERROR, mapping={'userId': 'pmCreator2'})
        self.assertEqual(messages.show()[-3].message, translate(msg))

    def test_checkAlreadySentToPloneMeeting(self):
        """Test in case we sent the element again to PloneMeeting, that should not happen...
           It check also that relevant annotation wipe out works correctly."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal)
        settings = ws4pmSettings.settings()
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        self._configureRequestForView(document)
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        view.proposingGroupId = 'developers'
        # create the 'pmCreator1' member area to be able to create an item
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator1')
        self.tool.getPloneMeetingFolder('plonegov-assembly', 'pmCreator1')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created member area...
        transaction.commit()
        # before sending, the element is not linked
        annotations = IAnnotations(document)
        self.assertFalse(WS4PMCLIENT_ANNOTATION_KEY in annotations)
        self.assertFalse(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                                                                           self.request.get('meetingConfigId')))
        # send the document
        self.assertTrue(view._doSendToPloneMeeting())
        # is linked to one item
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == [self.request.get('meetingConfigId'), ])
        self.assertTrue(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                        (self.request.get('meetingConfigId'),)))
        self.assertTrue(len(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})) == 1)
        messages = IStatusMessage(self.request)
        # there is one message saying that the item was correctly sent
        shownMessages = messages.show()
        self.assertEquals(shownMessages[-1].message, CORRECTLY_SENT_TO_PM_INFO)
        # call form again, it will display relevant status messages
        # the rendered form is u''
        self.assertTrue(settings.only_one_sending)
        self.assertTrue(view() == u'')
        # the item is not created again
        # is still linked to one item
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == [self.request.get('meetingConfigId'), ])
        self.assertTrue(len(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})) == 1)
        # a warning is displayed to the user
        self.request.response.status = 200  # if status in 300, messages are not deleted with show
        self.assertEquals(messages.show()[-1].message, ALREADY_SENT_TO_PM_ERROR)
        settings.only_one_sending = False
        self.assertFalse(settings.only_one_sending)
        view._finishedSent = False
        self.request.response.status = 200  # if status in 300, render is not called by z3cform
        self.assertIn('Send to PloneMeeting Assembly', view())
        self.assertEqual(len(messages.show()), 0)
        # if we remove the item in PloneMeeting, the view is aware of it
        itemUID = str(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})[0]['UID'])
        # avoid weird ConflictError while committing because of
        # self.portal._volatile_cache_keys PersistentMapping
        self.portal._volatile_cache_keys._p_changed = False
        transaction.commit()
        item = self.portal.uid_catalog(UID=itemUID)[0].getObject()
        # remove the item
        item.aq_inner.aq_parent.manage_delObjects(ids=[item.getId(), ])
        transaction.commit()
        # checkAlreadySentToPloneMeeting will wipe out inconsistent annotations
        # for now, annotations are inconsistent
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == [self.request.get('meetingConfigId'), ])
        self.assertFalse(
            view.ws4pmSettings.checkAlreadySentToPloneMeeting(
                document,
                (self.request.get('meetingConfigId'),)))
        # now it is consistent
        self.assertFalse(WS4PMCLIENT_ANNOTATION_KEY in annotations)
        self.assertTrue(len(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})) == 0)
        # the item can be sent again and will be linked to a new created item
        self.assertTrue(view._doSendToPloneMeeting())
        self.assertTrue(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                        (self.request.get('meetingConfigId'),)))
        self.assertTrue(len(ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})) == 1)
        # the item can also been send to another meetingConfig
        self.request.set('meetingConfigId', 'plonegov-assembly')
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        view.proposingGroupId = 'developers'
        self.assertFalse(
            view.ws4pmSettings.checkAlreadySentToPloneMeeting(
                document,
                (self.request.get('meetingConfigId'),)))
        self.request.form['form.widgets.category'] = [u'deployment', ]
        self.assertTrue(view._doSendToPloneMeeting())
        self.assertTrue(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                        (self.request.get('meetingConfigId'),)))
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == ['plonemeeting-assembly',
                                                                    self.request.get('meetingConfigId'), ])
        # if we remove the 2 items, a call to checkAlreadySentToPloneMeeting
        # without meetingConfigs will wipeout the annotations
        transaction.commit()
        itemUIDs = [str(elt['UID']) for elt in ws4pmSettings._soap_searchItems({'externalIdentifier': document.UID()})]

        item1 = self.portal.uid_catalog(UID=itemUIDs[0])[0].getObject()
        item2 = self.portal.uid_catalog(UID=itemUIDs[1])[0].getObject()
        item1.aq_inner.aq_parent.manage_delObjects(ids=[item1.getId(), ])
        item2.aq_inner.aq_parent.manage_delObjects(ids=[item2.getId(), ])
        transaction.commit()
        # annotations are still messed up
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == ['plonemeeting-assembly',
                                                                    self.request.get('meetingConfigId'), ])
        # wipe out annotations
        view.ws4pmSettings.checkAlreadySentToPloneMeeting(document)
        self.assertFalse(WS4PMCLIENT_ANNOTATION_KEY in annotations)

    def _configureRequestForView(self, obj):
        """Helper method that configure the request values so the view to test works..."""
        self.request.set('URL', obj.absolute_url())
        self.request.set('ACTUAL_URL', obj.absolute_url() + '/%s' % SEND_TO_PM_VIEW_NAME)
        self.request.set('meetingConfigId', 'plonemeeting-assembly')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testForms, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-
#
# File: testItemMethods.py
#
# GNU General Public License (GPL)
#

from imio.pm.ws.config import POD_TEMPLATE_ID_PATTERN
from imio.pm.wsclient.config import ANNEXID_MANDATORY_ERROR
from imio.pm.wsclient.config import CORRECTLY_SENT_TO_PM_INFO
from imio.pm.wsclient.config import FILENAME_MANDATORY_ERROR
from imio.pm.wsclient.config import MISSING_FILE_ERROR
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import UNABLE_TO_DETECT_MIMETYPE_ERROR
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import cleanMemoize
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from mock import Mock
from mock import patch
from Products.statusmessages.interfaces import IStatusMessage

import base64


class testViews(WS4PMCLIENTTestCase):
    """
        Tests the browser.settings SOAP client methods
    """
    def test_generateItemTemplateView(self):
        """
          Test the BrowserView that generates a given template of an item
        """
        self.changeUser('admin')
        messages = IStatusMessage(self.request)
        document = createDocument(self.portal)
        DOCUMENT_ABSOLUTE_URL = document.absolute_url()
        # we must obviously be connected to PM...
        view = document.restrictedTraverse('@@generate_document_from_plonemeeting')
        # nothing is generated, just redirected to the context
        self.assertFalse(view() != DOCUMENT_ABSOLUTE_URL)
        self.assertTrue(len(messages.show()) == 1)
        self.assertTrue(messages.show()[0].message == UNABLE_TO_CONNECT_ERROR)
        # _soap_connectToPloneMeeting is memoized...
        cleanMemoize(self.request)
        item = self._sendToPloneMeeting(document)
        # a statusmessage for having created the item successfully
        self.assertEqual(messages.show()[-1].message, CORRECTLY_SENT_TO_PM_INFO)
        view = document.restrictedTraverse('@@generate_document_from_plonemeeting')
        # with no templateFormat defined, the mimetype can not be determined
        # an error statusmessage is displayed
        # last added statusmessage
        # nothing is generated, just redirected to the context
        self.assertFalse(view() != DOCUMENT_ABSOLUTE_URL)
        self.assertEqual(messages.show()[-1].message, UNABLE_TO_DETECT_MIMETYPE_ERROR)
        self.request.set('templateFormat', 'odt')
        # if no templateFilename, an error is displayed, nothing is generated
        view = document.restrictedTraverse('@@generate_document_from_plonemeeting')
        # nothing is generated, just redirected to the context
        self.assertFalse(view() != DOCUMENT_ABSOLUTE_URL)
        self.assertEqual(messages.show()[-1].message, FILENAME_MANDATORY_ERROR)
        # if not valid itemUID defined, the item can not be found and so accessed
        self.request.set('templateFilename', 'filename')
        view = document.restrictedTraverse('@@generate_document_from_plonemeeting')
        # nothing is generated, just redirected to the context
        self.assertFalse(view() != DOCUMENT_ABSOLUTE_URL)
        self.assertEqual(
            messages.show()[-1].message, u"An error occured while generating the document in "
            "PloneMeeting!  The error message was : Server raised fault: 'You can not access this item!'"
        )
        # now with a valid itemUID but no valid templateId
        self.request.set('itemUID', item.UID())
        view = document.restrictedTraverse('@@generate_document_from_plonemeeting')
        # nothing is generated, just redirected to the context
        self.assertFalse(view() != DOCUMENT_ABSOLUTE_URL)
        self.assertEqual(
            messages.show()[-1].message, u"An error occured while generating the document in "
            "PloneMeeting!  The error message was : Server raised fault: 'You can not access this template!'"
        )
        # now with all valid infos
        self.request.set('templateId', POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt'))
        view = document.restrictedTraverse('@@generate_document_from_plonemeeting')
        res = view()
        # with have a real result, aka not redirected to the context, a file
        self.assertTrue(view() != DOCUMENT_ABSOLUTE_URL)
        self.assertTrue(len(res) > 10000)
        self.assertEquals(self.request.response.headers,
                          {'content-type': 'application/vnd.oasis.opendocument.text',
                           'location': '{0}/document'.format(self.portal.absolute_url()),
                           'content-disposition': 'inline;filename="filename.odt"'})

    def test_DownloadAnnexFromItemView_without_annex_id(self):
        """
          Test the BrowserView that return the annex of an item
        """
        messages = IStatusMessage(self.request)
        self.changeUser('admin')
        download_annex = self.portal.restrictedTraverse('@@download_annex_from_plonemeeting')
        # mock connexion to PloneMeeting
        download_annex.ws4pmSettings._soap_connectToPloneMeeting = Mock(return_value=True)
        download_annex()
        self.assertEqual(messages.show()[-1].message, ANNEXID_MANDATORY_ERROR)

    @patch('imio.pm.wsclient.browser.settings.WS4PMClientSettings._soap_getItemInfos')
    def test_DownloadAnnexFromItemView_with_no_result(self, _soap_getItemInfos):
        """
          Test the BrowserView that return the annex of an item
        """
        # return no annex
        _soap_getItemInfos.return_value = None
        messages = IStatusMessage(self.request)
        self.changeUser('admin')
        document = createDocument(self.portal)
        self.request.set('annex_id', 'my_annex')
        download_annex = document.restrictedTraverse('@@download_annex_from_plonemeeting')
        # mock connexion to PloneMeeting
        download_annex.ws4pmSettings._soap_connectToPloneMeeting = Mock(return_value=True)
        download_annex()
        self.assertEqual(messages.show()[-1].message, MISSING_FILE_ERROR)

    @patch('imio.pm.wsclient.browser.settings.WS4PMClientSettings._soap_getItemInfos')
    def test_DownloadAnnexFromItemView(self, _soap_getItemInfos):
        """
          Test the BrowserView that return the annex of an item
        """
        # return an annex
        annex_id = 'my_annex'
        _soap_getItemInfos.return_value = [type(
            'ItemInfo', (object,), {
                'annexes': [
                    type(
                        'AnnexInfo', (object,), {
                            'id': annex_id,
                            'filename': 'my_annex.pdf',
                            'file': base64.b64encode('Hello!')
                        }
                    )
                ]
            }
        )]
        self.changeUser('admin')
        document = createDocument(self.portal)
        self.request.set('annex_id', annex_id)
        download_annex = document.restrictedTraverse('@@download_annex_from_plonemeeting')
        # mock connexion to PloneMeeting
        download_annex.ws4pmSettings._soap_connectToPloneMeeting = Mock(return_value=True)
        annex = download_annex()
        self.assertEqual(annex, 'Hello!')
        self.assertEqual(self.request.RESPONSE.headers.get('content-type'), 'application/pdf')
        self.assertEqual(
            self.request.RESPONSE.headers.get('content-disposition'),
            'inline;filename="my_annex.pdf"'
        )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testViews, prefix='test_'))
    return suite



# suffix used when adding our actions to portal_actions
ACTION_SUFFIX = 'plonemeeting_wsclient_action_'

# taken from imio.pm.ws
DEFAULT_NO_WARNING_MESSAGE = 'There was NO WARNING message during item creation.'

# messages
CAN_NOT_SEE_LINKED_ITEMS_INFO = u"This element is linked to item(s) in PloneMeeting but your are not allowed to see it."
CORRECTLY_SENT_TO_PM_INFO = u"The item has been correctly sent to PloneMeeting."
CONFIG_UNABLE_TO_CONNECT_ERROR = u"Unable to connect to PloneMeeting! The error message was : ${error}!"
UNABLE_TO_CONNECT_ERROR = u"Unable to connect to PloneMeeting! Please contact system administrator!"
ALREADY_SENT_TO_PM_ERROR = u"This element has already been sent to PloneMeeting!"
TAL_EVAL_FIELD_ERROR = u"There was an error evaluating the TAL expression '${expr}' for the field '${field_name}'! " \
                       "The error was : ${error}. Please contact system administrator."
UNABLE_TO_DETECT_MIMETYPE_ERROR = u"Could not detect correct mimetype for item template! "\
                                  "Please contact system administrator!"
FILENAME_MANDATORY_ERROR = u"A filename is mandatory while generating a document! "\
                           "Please contact system administrator!"
UNABLE_TO_DISPLAY_VIEWLET_ERROR = u"Unable to display informations about the potentially linked item " \
    "in PloneMeeting because there was an error evaluating the TAL expression '${expr}' " \
    "for the field '${field_name}'! The error was : '${error}'.  Please contact system administrator."
CONFIG_CREATE_ITEM_PM_ERROR = u"An error occured during the item creation in PloneMeeting! " \
                              "The error message was : ${error}"
NO_PROPOSING_GROUP_ERROR = u"The configuration specify that user '${userId}' will create the item in PloneMeeting " \
                           "but this user can not create item for any proposingGroup in PloneMeeting!"
NO_USER_INFOS_ERROR = u"Could not get userInfos in PloneMeeting for user '${userId}'!"
NO_FIELD_MAPPINGS_ERROR = u"No field_mappings defined in the WS4PMClient configuration!"
CAN_NOT_CREATE_FOR_PROPOSING_GROUP_ERROR = u"The current user can not create an item with the proposingGroup forced " \
                                           "thru the configuration! Please contact system administrator!"
NO_CONFIG_INFOS_ERROR = u"No configuration informations found!"
CAN_NOT_CREATE_WITH_CATEGORY_ERROR = u"The current user can not create an item with the category forced " \
                                     "thru the configuration! Please contact system administrator!"
SEND_WITHOUT_SUFFICIENT_FIELD_MAPPINGS_DEFINED_WARNING = u"No sufficient field mappings are defined in the " \
                                                         "configuration. It is recommended to define at least the " \
                                                         "'title' mapping, but 'description' and 'decision' should " \
                                                         "also be defined. It will ne be possible to create the " \
                                                         "item in PloneMeeting."
ANNEXID_MANDATORY_ERROR = u"An annex id is mandatory to download an annex!"
MISSING_FILE_ERROR = u"The requested file could not be found on the item"

# annotations key
WS4PMCLIENT_ANNOTATION_KEY = "imio.pm.wsclient-sent_to"



# -*- coding: utf-8 -*-

from collective.z3cform.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow
from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import ACTION_SUFFIX
from imio.pm.wsclient.config import CONFIG_CREATE_ITEM_PM_ERROR
from imio.pm.wsclient.config import CONFIG_UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.memoize.view import memoize
from plone.registry.interfaces import IRecordModifiedEvent
from plone.registry.interfaces import IRegistry
from Products.CMFCore.ActionInformation import Action
from Products.CMFCore.Expression import createExprContext
from Products.CMFCore.Expression import Expression
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage
from suds.client import Client
from suds.transport.http import HttpAuthenticated
from suds.xsd.doctor import Import
from suds.xsd.doctor import ImportDoctor
from z3c.form import button
from z3c.form import field
from zope import schema
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory


class IGeneratedActionsSchema(Interface):
    """Schema used for the datagrid field 'generated_actions' of IWS4PMClientSettings."""
    condition = schema.TextLine(
        title=_("TAL Condition"),
        required=False,)
    permissions = schema.Choice(
        title=_("Permissions"),
        required=False,
        vocabulary=u'imio.pm.wsclient.possible_permissions_vocabulary')
    pm_meeting_config_id = schema.Choice(
        title=_("PloneMeeting meetingConfig id"),
        required=True,
        vocabulary=u'imio.pm.wsclient.pm_meeting_config_id_vocabulary')


class IFieldMappingsSchema(Interface):
    """Schema used for the datagrid field 'field_mappings' of IWS4PMClientSettings."""
    field_name = schema.Choice(
        title=_("PloneMeeting field name"),
        required=True,
        vocabulary=u'imio.pm.wsclient.pm_item_data_vocabulary')
    expression = schema.TextLine(
        title=_("TAL expression to evaluate for the corresponding PloneMeeting field name"),
        required=True,)


class IAllowedAnnexTypesSchema(Interface):
    """Schema used for the datagrid field 'allowed_annex_type' of IWS4PMClientSettings."""
    annex_type = schema.TextLine(
        title=_("Annex type"),
        required=True)


class IUserMappingsSchema(Interface):
    """Schema used for the datagrid field 'user_mappings' of IWS4PMClientSettings."""
    local_userid = schema.TextLine(
        title=_("Local user id"),
        required=True)
    pm_userid = schema.TextLine(
        title=_("PloneMeeting corresponding user id"),
        required=True,)


class IWS4PMClientSettings(Interface):
    """
    Configuration of the WS4PM Client
    """
    pm_url = schema.TextLine(
        title=_(u"PloneMeeting WSDL URL"),
        required=True,)
    pm_timeout = schema.Int(
        title=_(u"PloneMeeting connection timeout"),
        description=_(u"Enter the timeout while connecting to PloneMeeting. Do not set a too high timeout because it "
                      "will impact the load of the viewlet showing PM infos on a sent element if PM is not available. "
                      "Default is '10' seconds."),
        default=10,
        required=True,)
    pm_username = schema.TextLine(
        title=_("PloneMeeting username to use"),
        description=_(u"The user must be at least a 'MeetingManager'. Nevertheless, items will be created regarding "
                      "the <i>User ids mappings</i> defined here under."),
        required=True,)
    pm_password = schema.Password(
        title=_("PloneMeeting password to use"),
        required=True,)
    only_one_sending = schema.Bool(
        title=_("An element can be sent one time only"),
        default=True,
        required=True,)
    viewlet_display_condition = schema.TextLine(
        title=_("Viewlet display condition"),
        description=_("Enter a TAL expression that will be evaluated to check if the viewlet displaying "
                      "informations about the created items in PloneMeeting should be displayed. "
                      "If empty, the viewlet will only be displayed if an item is actually linked to it. "
                      "The 'isLinked' variable representing this default behaviour is available "
                      "in the TAL expression."),
        required=False,)
    field_mappings = schema.List(
        title=_("Field accessor mappings"),
        description=_("For every available data you can send, define in the mapping a TAL expression that will be "
                      "executed to obtain the correct value to send. The 'meetingConfigId' and 'proposingGroupId' "
                      "variables are also available for the expression. Special case for the 'proposingGroup' and "
                      "'category' fields, you can 'force' the use of a particular value by defining it here. If not "
                      "defined the user will be able to use every 'proposingGroup' or 'category' he is allowed to "
                      "use in PloneMeeting."),
        value_type=DictRow(title=_("Field mappings"),
                           schema=IFieldMappingsSchema,
                           required=False),
        required=False,)
    allowed_annexes_types = schema.List(
        title=_("Allowed annexes types"),
        description=_("List here the annexes types allowed to be display in the linked meeting item viewlet"),
        value_type=DictRow(title=_("Allowed annex type"),
                           schema=IAllowedAnnexTypesSchema,
                           required=False),
        default=[],
        required=False,)
    user_mappings = schema.List(
        title=_("User ids mappings"),
        description=_("By default, while sending an element to PloneMeeting, the user id of the logged in user "
                      "is used and a binding is made to the same user id in PloneMeeting. "
                      "If the local user id does not exist in PloneMeeting, you can define here the user mappings "
                      "to use. For example : 'jdoe' in 'Local user id' of the current application correspond to "
                      "'johndoe' in PloneMeeting."),
        value_type=DictRow(title=_("User mappings"),
                           schema=IUserMappingsSchema,
                           required=False),
        required=False,)
    generated_actions = schema.List(
        title=_("Generated actions"),
        description=_("Actions to send an item to PloneMeeting can be generated. First enter a 'TAL condition' "
                      "evaluated to show the action then choose permission(s) the user must have to see the action. "
                      "Finally, choose the meetingConfig the item will be sent to."),
        value_type=DictRow(title=_("Actions"),
                           schema=IGeneratedActionsSchema,
                           required=False),
        required=False,)


class WS4PMClientSettingsEditForm(RegistryEditForm):
    """
    Define form logic
    """
    schema = IWS4PMClientSettings
    label = _(u"WS4PM Client settings")
    description = _(u"""""")

    fields = field.Fields(IWS4PMClientSettings)
    fields['generated_actions'].widgetFactory = DataGridFieldFactory
    fields['field_mappings'].widgetFactory = DataGridFieldFactory
    fields['allowed_annexes_types'].widgetFactory = DataGridFieldFactory
    fields['user_mappings'].widgetFactory = DataGridFieldFactory

    def updateFields(self):
        super(WS4PMClientSettingsEditForm, self).updateFields()
        portal = getSite()
        # this is also called by the kss inline_validation, avoid too much work...
        if not portal.__module__ == 'Products.CMFPlone.Portal':
            return
        ctrl = getMultiAdapter((portal, portal.REQUEST), name='ws4pmclient-settings')
        # if we can not getConfigInfos from the given pm_url, we do not permit to edit other parameters
        generated_actions_field = self.fields.get('generated_actions')
        field_mappings = self.fields.get('field_mappings')
        if not ctrl._soap_getConfigInfos():
            generated_actions_field.mode = 'display'
            field_mappings.mode = 'display'
        else:
            if generated_actions_field.mode == 'display' and \
               'form.buttons.save' not in self.request.form.keys():
                # only change mode while not in the "saving" process (that calls updateFields, but why?)
                # because it leads to loosing generated_actions because a [] is returned by extractDate here above
                self.fields.get('generated_actions').mode = 'input'
                self.fields.get('field_mappings').mode = 'input'

    def updateWidgets(self):
        super(WS4PMClientSettingsEditForm, self).updateWidgets()

    @button.buttonAndHandler(_('Save'), name=None)
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        self.applyChanges(data)
        IStatusMessage(self.request).addStatusMessage(_(u"Changes saved"),
                                                      "info")
        self.context.REQUEST.RESPONSE.redirect("@@ws4pmclient-settings")

    @button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        IStatusMessage(self.request).addStatusMessage(_(u"Edit cancelled"),
                                                      "info")
        self.request.response.redirect("%s/%s" % (self.context.absolute_url(),
                                                  self.control_panel_view))


class WS4PMClientSettings(ControlPanelFormWrapper):
    form = WS4PMClientSettingsEditForm

    @memoize
    def settings(self):
        """ """
        registry = queryUtility(IRegistry)
        settings = registry.forInterface(IWS4PMClientSettings, check=False)
        return settings

    @memoize
    def _soap_connectToPloneMeeting(self):
        """
          Connect to distant PloneMeeting.
          Either return None or the connected client.
        """
        settings = self.settings()
        url = self.request.form.get('form.widgets.pm_url') or settings.pm_url or ''
        username = self.request.form.get('form.widgets.pm_username') or settings.pm_username or ''
        password = self.request.form.get('form.widgets.pm_password') or settings.pm_password or ''
        timeout = self.request.form.get('form.widgets.pm_timeout') or settings.pm_timeout or ''
        imp = Import('http://schemas.xmlsoap.org/soap/encoding/')
        d = ImportDoctor(imp)
        t = HttpAuthenticated(username=username, password=password)
        try:
            client = Client(url, doctor=d, transport=t, timeout=int(timeout))
            # call a SOAP server test method to check that everything is fine with given parameters
            client.service.testConnection('')
        except Exception, e:
            # if we are really on the configuration panel, display relevant message
            if self.request.get('URL', '').endswith('@@ws4pmclient-settings'):
                IStatusMessage(self.request).addStatusMessage(
                    _(CONFIG_UNABLE_TO_CONNECT_ERROR, mapping={'error': (e.message or str(e.reason))}), "error")
            return None
        return client

    def _soap_checkIsLinked(self, data):
        """Query the checkIsLinked SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            return client.service.checkIsLinked(**data)

    @memoize
    def _soap_getConfigInfos(self, showCategories=False):
        """Query the getConfigInfos SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            return client.service.getConfigInfos(showCategories=showCategories)

    @memoize
    def _soap_getUserInfos(self, showGroups=False, suffix=''):
        """Query the getUserInfos SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            # get the inTheNameOf userid if it was not already set
            userId = self._getUserIdToUseInTheNameOfWith(mandatory=True)
            try:
                return client.service.getUserInfos(userId, showGroups, suffix)
            except Exception:
                return None

    def _soap_searchItems(self, data):
        """Query the searchItems SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            # get the inTheNameOf userid if it was not already set
            if 'inTheNameOf' not in data:
                data['inTheNameOf'] = self._getUserIdToUseInTheNameOfWith()
            return client.service.searchItems(**data)

    def _soap_getItemInfos(self, data):
        """Query the getItemInfos SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            # get the inTheNameOf userid if it was not already set
            if 'inTheNameOf' not in data:
                data['inTheNameOf'] = self._getUserIdToUseInTheNameOfWith()
            return client.service.getItemInfos(**data)

    def _soap_getMeetingsAcceptingItems(self, data):
        """Query the getItemInfos SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            if 'inTheNameOf' not in data:
                data['inTheNameOf'] = self._getUserIdToUseInTheNameOfWith()
            return client.service.meetingsAcceptingItems(**data)

    def _soap_getItemTemplate(self, data):
        """Query the getItemTemplate SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            if 'inTheNameOf' not in data:
                data['inTheNameOf'] = self._getUserIdToUseInTheNameOfWith()
            try:
                return client.service.getItemTemplate(**data)
            except Exception, exc:
                IStatusMessage(self.request).addStatusMessage(
                    _(u"An error occured while generating the document in PloneMeeting!  "
                      "The error message was : %s" % exc), "error")

    @memoize
    def _soap_getItemCreationAvailableData(self):
        """Query SOAP WSDL to obtain the list of available fields useable while creating an item."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            # extract data from the CreationData ComplexType that is used to create an item
            namespace = str(client.wsdl.tns[1])
            res = ['proposingGroup']
            res += [str(data.name) for data in
                    client.factory.wsdl.build_schema().types['CreationData', namespace].rawchildren[0].rawchildren]
            return res

    def _soap_createItem(self, meetingConfigId, proposingGroupId, creationData):
        """Query the createItem SOAP server method."""
        client = self._soap_connectToPloneMeeting()
        if client is not None:
            try:
                # we create an item inTheNameOf the currently connected member
                # _getUserIdToCreateWith returns None if the settings defined username creates the item
                inTheNameOf = self._getUserIdToUseInTheNameOfWith()
                res = client.service.createItem(meetingConfigId,
                                                proposingGroupId,
                                                creationData,
                                                inTheNameOf=inTheNameOf)
                # return 'UID' and 'warnings' if any current user is a Manager
                warnings = []
                if self.context.portal_membership.getAuthenticatedMember().has_role('Manager'):
                    warnings = 'warnings' in res.__keylist__ and res['warnings'] or []
                return res['UID'], warnings
            except Exception, exc:
                IStatusMessage(self.request).addStatusMessage(_(CONFIG_CREATE_ITEM_PM_ERROR, mapping={'error': exc}),
                                                              "error")

    def _getUserIdToUseInTheNameOfWith(self, mandatory=False):
        """
          Returns the userId that will actually create the item.
          Returns None if we found out that it is the defined settings.pm_username
          that will create the item : either it is the currently connected user,
          or there is an existing user_mapping between currently connected user
          and settings.pm_username user.
          If p_mandatory is True, returns mndatorily a userId.
        """
        member = self.context.portal_membership.getAuthenticatedMember()
        memberId = member.getId()
        # get username specified to connect to the SOAP distant site
        settings = self.settings()
        soapUsername = settings.pm_username and settings.pm_username.strip()
        # if current user is the user defined in the settings, return None
        if memberId == soapUsername:
            if mandatory:
                return soapUsername
            else:
                return None
        # check if a user_mapping exists
        if settings.user_mappings:
            for user_mapping in settings.user_mappings:
                localUserId, distantUserId = user_mapping['local_userid'], user_mapping['pm_userid']
                # if we found a mapping for the current user, check also
                # that the distantUserId the mapping is linking to, is not the soapUsername
                if memberId == localUserId.strip():
                    if not soapUsername == distantUserId.strip():
                        return distantUserId.strip()
                    else:
                        if mandatory:
                            return soapUsername
                        else:
                            return None
        return memberId

    def checkAlreadySentToPloneMeeting(self, context, meetingConfigIds=[]):
        """
          Check if the element has already been sent to PloneMeeting to avoid double sents
          If an item needs to be doubled in PloneMeeting, it is PloneMeeting's duty
          If p_meetingConfigIds is empty (), then it checks every available meetingConfigId it was sent to...
          The script will return :
          - 'None' if could not connect to PloneMeeting
          - True if the p_context is linked to an item of p_meetingConfigIds
          - False if p_context is not linked to an item of p_meetingConfigIds
          This script also wipe out every meetingConfigIds for wich the item does not exist anymore in PloneMeeting
        """
        annotations = IAnnotations(context)
        # for performance reason (avoid to connect to SOAP if no annotations)
        # if there are no relevant annotations, it means that the p_context
        # is not linked and we return False
        isLinked = False
        if WS4PMCLIENT_ANNOTATION_KEY in annotations:
            # the item seems to have been sent, but double check in case it was
            # deleted in PloneMeeting after having been sent
            # warning, here searchItems inTheNameOf the super user to be sure
            # that we can access it in PloneMeeting
            if not meetingConfigIds:
                # evaluate the meetingConfigIds in the annotation
                # this will wipe out the entire annotation
                meetingConfigIds = list(annotations[WS4PMCLIENT_ANNOTATION_KEY])
            for meetingConfigId in meetingConfigIds:
                res = self._soap_checkIsLinked({'externalIdentifier': context.UID(),
                                                'meetingConfigId': meetingConfigId, })
                # if res is None, it means that it could not connect to PloneMeeting
                if res is None:
                    return None
                # we found at least one linked item
                elif res is True:
                    isLinked = True
                # could connect to PM but did not find a result
                elif res is False:
                    # either the item was deleted in PloneMeeting
                    # or it was never send, wipe out if it was deleted in PloneMeeting
                    if meetingConfigId in annotations[WS4PMCLIENT_ANNOTATION_KEY]:
                        # do not use .remove directly on the annotations or it does not save
                        # correctly and when Zope restarts, the removed annotation is still there???
                        existingAnnotations = list(annotations[WS4PMCLIENT_ANNOTATION_KEY])
                        existingAnnotations.remove(meetingConfigId)
                        annotations[WS4PMCLIENT_ANNOTATION_KEY] = existingAnnotations
                    if not annotations[WS4PMCLIENT_ANNOTATION_KEY]:
                        # remove the entire annotation key if empty
                        del annotations[WS4PMCLIENT_ANNOTATION_KEY]
        return isLinked

    def renderTALExpression(self, context, portal, expression, vars={}):
        """
          Renders given TAL expression in p_expression.
          p_vars contains extra variables that will be done available in the TAL expression to render
        """
        res = ''
        if expression:
            expression = expression.strip()
            ctx = createExprContext(context.aq_inner.aq_parent, portal, context)
            vars['context'] = context
            ctx.vars.update(vars)
            for k, v in vars.items():
                ctx.setContext(k, v)
            res = Expression(expression)(ctx)
        # make sure we do not return None because it breaks SOAP call
        if res is None:
            return ''
        else:
            return safe_unicode(res)

    def getMeetingConfigTitle(self, meetingConfigId):
        """
          Return the title of the given p_meetingConfigId
          Use the vocabulary u'imio.pm.wsclient.pm_meeting_config_id_vocabulary'
        """
        # get the pm_meeting_config_id_vocabulary so we will be able to displayValue
        factory = queryUtility(IVocabularyFactory, u'imio.pm.wsclient.pm_meeting_config_id_vocabulary')
        # self.context is portal
        meetingConfigVocab = factory(self.context)
        try:
            return meetingConfigVocab.getTerm(meetingConfigId).title
        except LookupError:
            return ''


def notify_configuration_changed(event):
    """Event subscriber that is called every time the configuration changed."""
    portal = getSite()

    if IRecordModifiedEvent.providedBy(event):
        # generated_actions changed, we need to update generated actions in portal_actions
        if event.record.fieldName == 'generated_actions':
            # if generated_actions have been changed, remove every existing generated_actions then recreate them
            # first remove every actions starting with ACTION_SUFFIX
            object_buttons = portal.portal_actions.object_buttons
            for object_button in object_buttons.objectValues():
                if object_button.id.startswith(ACTION_SUFFIX):
                    object_buttons.manage_delObjects([object_button.id])
            # then recreate them
            i = 1
            ws4pmSettings = getMultiAdapter((portal, portal.REQUEST), name='ws4pmclient-settings')
            for actToGen in event.record.value:
                actionId = "%s%d" % (ACTION_SUFFIX, i)
                action = Action(
                    actionId,
                    title=translate(
                        'Send to',
                        domain='imio.pm.wsclient',
                        mapping={
                            'meetingConfigTitle':
                            ws4pmSettings.getMeetingConfigTitle(actToGen['pm_meeting_config_id']),
                        },
                        context=portal.REQUEST),
                    description='', i18n_domain='imio.pm.wsclient',
                    url_expr='string:${object_url}/@@send_to_plonemeeting_form?meetingConfigId=%s'
                             % actToGen['pm_meeting_config_id'],
                    icon_expr='string:${portal_url}/++resource++imio.pm.wsclient.images/send_to_plonemeeting.png',
                    available_expr=actToGen['condition'] or '',
                    # make sure we have a tuple as permissions value
                    permissions=actToGen['permissions'] and (actToGen['permissions'], ) or ('View', ),
                    visible=True)
                object_buttons._setObject(actionId, action)
                i = i + 1



from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class pm_meeting_config_id_vocabulary(object):
    """
      Overrides the existing imio.pm.wsclient.browser.vocabularies.pm_meeting_config_id_vocabulary
      vocabulary that needs PloneMeeting to work and in the testSettings.py we do not have PloneMeeting...
      For other tests, the existing vocabulary is used.
    """
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Query every existing MeetingConfigs in a distant PloneMeeting."""
        # query existing MeetingGroups from distant PM site if the default_pm_url is defined and working
        terms = []
        terms.append(SimpleTerm(u'plonegov-assembly', u'plonegov-assembly', u'PloneGov Assembly', ))
        terms.append(SimpleTerm(u'plonemeeting-assembly', u'plonemeeting-assembly', u'PloneMeeting Assembly', ))
        return SimpleVocabulary(terms)


pm_meeting_config_id_vocabularyFactory = pm_meeting_config_id_vocabulary()



# -*- coding: utf-8 -*-
#
# File: testEvents.py
#
# GNU General Public License (GPL)
#

from imio.pm.wsclient.interfaces import ISentToPMEvent
from imio.pm.wsclient.interfaces import IWillbeSendToPMEvent
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import SEND_TO_PM_VIEW_NAME
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from zope.component import getGlobalSiteManager
from zope.interface import Interface

import transaction


class FailTest(Exception):
    """ Raised when the ws event is not notified at the right moment """


class testEvents(WS4PMCLIENTTestCase):
    """
        Tests the browser.settings SOAP client methods
    """

    def setUp(self):
        """
           Set a ws sending view.
        """
        super(testEvents, self).setUp()
        self.sending_view = self._setup_sending_view()

    def test_WillBeSendToPM_event(self):
        """
            Test notification of WillbeSendToPM event when calling _doSendToPloneMeeting.
        """
        sending_view = self.sending_view

        # register a handler raising a dummy exception for this event
        class WillBeSendToPm(Exception):
            """ """
        def will_be_send_to_pm_handler(obj, event):
            if not sending_view._finishedSent:
                raise WillBeSendToPm
            raise FailTest("the WillBeSendToPm event should be raised before sending the item to PM")

        gsm = getGlobalSiteManager()
        gsm.registerHandler(will_be_send_to_pm_handler, (Interface, IWillbeSendToPMEvent))

        # send the element to pm, the event handler should raise the exception
        with self.assertRaises(WillBeSendToPm):
            self.sending_view._doSendToPloneMeeting()

        gsm.unregisterHandler(will_be_send_to_pm_handler, (Interface, IWillbeSendToPMEvent))

    def test_SentToPM_event(self):
        """
            Test notification of SentToPM event when calling _doSendToPloneMeeting.
        """
        sending_view = self.sending_view

        # register a handler raising a dummy exception for this event
        class SentToPm(Exception):
            """ """
        def sent_to_pm_handler(obj, event):
            if sending_view._finishedSent:
                raise SentToPm
            raise FailTest("the SentToPm event should be raised after sending the item to PM")

        gsm = getGlobalSiteManager()
        gsm.registerHandler(sent_to_pm_handler, (Interface, ISentToPMEvent))

        # send the element to pm, the event handler should raise the exception
        with self.assertRaises(SentToPm):
            self.sending_view._doSendToPloneMeeting()

        gsm.unregisterHandler(sent_to_pm_handler, (Interface, ISentToPMEvent))

    def _setup_sending_view(self):
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        self.request.set('URL', document.absolute_url())
        self.request.set('ACTUAL_URL', document.absolute_url() + '/%s' % SEND_TO_PM_VIEW_NAME)
        self.request.set('meetingConfigId', 'plonemeeting-assembly')
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator1')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created member area...
        transaction.commit()
        view.proposingGroupId = 'developers'
        return view


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testEvents, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-

from dateutil import tz
from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import CAN_NOT_SEE_LINKED_ITEMS_INFO
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import UNABLE_TO_DISPLAY_VIEWLET_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from plone.app.layout.viewlets.common import ViewletBase
from plone.memoize.instance import memoize
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.schema.interfaces import IVocabularyFactory


class PloneMeetingInfosViewlet(ViewletBase):
    """This viewlet display informations from PloneMeeting if the current object has been 'sent' to it.
       This viewlet will be displayed only if there are informations to show."""

    index = ViewPageTemplateFile('templates/plonemeeting_infos.pt')

    def update(self):
        self.portal_state = getMultiAdapter((self.context, self.request),
                                            name=u'plone_portal_state')
        self.ws4pmSettings = getMultiAdapter((self.portal_state.portal(), self.request), name='ws4pmclient-settings')

    @memoize
    def available(self):
        """
          Check if the viewlet is available and needs to be shown.
          This method returns either True or False, or a tuple of str
          that contains an information message (str1 is the translated message
          and str2 is the message type : info, error, warning).
        """
        # if we have an annotation specifying that the item was sent, we show the viewlet
        settings = self.ws4pmSettings.settings()
        isLinked = self.ws4pmSettings.checkAlreadySentToPloneMeeting(self.context)
        # in case it could not connect to PloneMeeting, checkAlreadySentToPloneMeeting returns None
        if isLinked is None:
            return (_(UNABLE_TO_CONNECT_ERROR), 'error')
        viewlet_display_condition = settings.viewlet_display_condition
        # if we have no defined viewlet_display_condition, use the isLinked value
        if not viewlet_display_condition or not viewlet_display_condition.strip():
            return isLinked
        # add 'isLinked' to data available in the TAL expression
        vars = {}
        vars['isLinked'] = isLinked
        try:
            res = self.ws4pmSettings.renderTALExpression(self.context,
                                                         self.portal_state.portal(),
                                                         settings.viewlet_display_condition,
                                                         vars)
            if not res:
                return False
        except Exception, e:
            return (_(UNABLE_TO_DISPLAY_VIEWLET_ERROR, mapping={'expr': settings.viewlet_display_condition,
                                                                'field_name': 'viewlet_display_condition',
                                                                'error': e}), 'error')
        # evaluate self.getPloneMeetingLinkedInfos
        linkedInfos = self.getPloneMeetingLinkedInfos()
        if isinstance(linkedInfos, tuple):
            # if self.getPloneMeetingLinkedInfos has errors, it returns
            # also a tuple with error message
            return linkedInfos
        return True

    @memoize
    def getPloneMeetingLinkedInfos(self):
        """Search items created for context.
           To get every informations we need, we will use getItemInfos(showExtraInfos=True)
           because we need the meetingConfig id and title...
           So search the items with searchItems then query again each found items
           with getConfigInfos.
           If we encounter an error, we return a tuple as 'usual' like in self.available"""
        try:
            items = self.ws4pmSettings._soap_searchItems({'externalIdentifier': self.context.UID()})
        except Exception, exc:
            return (_(u"An error occured while searching for linked items in PloneMeeting!  "
                      "The error message was : %s" % exc), 'error')
        # if we are here, it means that the current element is actually linked to item(s)
        # in PloneMeeting but the current user can not see it!
        if not items:
            # we return a message in a tuple
            return (_(CAN_NOT_SEE_LINKED_ITEMS_INFO), 'info')

        annotations = IAnnotations(self.context)
        sent_to = annotations[WS4PMCLIENT_ANNOTATION_KEY]
        res = []
        # to be able to know if some infos in PloneMeeting where not found
        # for current user, save the infos actually shown...
        settings = self.ws4pmSettings.settings()
        allowed_annexes_types = [line.values()[0] for line in settings.allowed_annexes_types]
        shownItemsMeetingConfigId = []
        for item in items:
            res.append(self.ws4pmSettings._soap_getItemInfos({'UID': item['UID'],
                                                              'showExtraInfos': True,
                                                              'showAnnexes': True,
                                                              'allowed_annexes_types': allowed_annexes_types,
                                                              'include_annex_binary': False,
                                                              'showExtraInfos': True,
                                                              'showTemplates': True})[0])
            lastAddedItem = res[-1]
            shownItemsMeetingConfigId.append(lastAddedItem['extraInfos']['meeting_config_id'])
            # XXX special case if something went wrong and there is an item in PM
            # that is not in the context sent_to annotation
            lastAddedItemMeetingConfigId = str(lastAddedItem['extraInfos']['meeting_config_id'])
            if lastAddedItemMeetingConfigId not in sent_to:
                existingSentTo = list(sent_to)
                existingSentTo.append(lastAddedItemMeetingConfigId)
                annotations[WS4PMCLIENT_ANNOTATION_KEY] = existingSentTo
                sent_to = annotations[WS4PMCLIENT_ANNOTATION_KEY]

        # if the number of items found is inferior to elements sent, it means
        # that some infos are not viewable by current user, we add special message
        if not len(items) == len(sent_to):
            # get meetingConfigs infos, use meetingConfig vocabulary
            factory = queryUtility(IVocabularyFactory, u'imio.pm.wsclient.pm_meeting_config_id_vocabulary')
            meetingConfigVocab = factory(self.portal_state.portal())
            # add special result
            for sent in annotations[WS4PMCLIENT_ANNOTATION_KEY]:
                if sent not in shownItemsMeetingConfigId:
                    # append a special result : nothing else but the meeting_config_id and title
                    # in extraInfos so sort here under works correctly
                    # in the linked viewlet template, we test if there is a 'UID' in the given infos, if not
                    # it means that it is this special message
                    res.append({'extraInfos': {'meeting_config_id': sent,
                                               'meeting_config_title': meetingConfigVocab.getTerm(sent).title}})

        # sort res to comply with sent order, for example sent first to college then council
        def sortByMeetingConfigId(x, y):
            return cmp(sent_to.index(x['extraInfos']['meeting_config_id']),
                       sent_to.index(y['extraInfos']['meeting_config_id']))
        res.sort(sortByMeetingConfigId)
        return res

    def displayMeetingDate(self, meeting_date):
        """Display a correct related meeting date :
           - if linked to a meeting, either '-'
           - manage displayed hours (hide hours if 00:00)"""
        if meeting_date.year == 1950:
            return '-'

        # now determinate result of toLocalizedTime before calling it...
        # we will just check if given p_meeting_date that is UTC would have
        # his hour to 0 after being localized to relevant timezone (what toLocalizedTime does)
        # localize meetingDate because it does not work with naive dates
        localMeetingDate = meeting_date.replace(tzinfo=tz.tzlocal())
        delta = localMeetingDate.utcoffset()
        utcMeetingDate = localMeetingDate - delta
        # set utcMeetingDate as being UTC
        utcMeetingDate = utcMeetingDate.replace(tzinfo=tz.tzutc())
        # if hour is 0, hide it, so call toLocalizedTime with long_format=False
        if utcMeetingDate.astimezone(tz.tzlocal()).hour == 0:
            long_format = False
        else:
            long_format = True
        return self.context.restrictedTraverse('@@plone').toLocalizedTime(meeting_date, long_format=long_format)
